/*
 * [HunkIO.java]
 *
 * Summary: read/write a file in one large hunk.
 *
 * Copyright: (c) 2002-2013 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.6+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.3 2005-06-18 add more consistent bat files.
 *  1.4 2006-01-14 readEntire file now works with multi-byte
 *                 encodings.
 *  1.5 2006-01-25 readFile/writeFile with non-default
 *                 encoding, raw read/write entire file.
 *  1.6 2007-05-24 add icon, pad, Intellij inspector.
 *  1.7 2009-05-03 create PrintWriterPlus vs PrintWriterPortable
 *  1.8 2013-02-19 convert all encoding parms from String to Charset, all file parms form String to File
 */
package com.mindprod.hunkio;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;

/**
 * read/write a file in one large hunk.
 * <p/>
 * Variants to read/write bytes, strings with default encoding and strings with specified encoding.
 * No main method.
 * We don't use java.nio.charset.Charset because it requires JDK 1.4+.
 * See also the more elaborate filetransfer package.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.8 2013-02-19 convert all encoding parms from String to Charset, all file parms form String to File
 * @since 2002-07-12
 */
public final class HunkIO
    {
    // ------------------------------ CONSTANTS ------------------------------

    /**
     * true to include the debugging harness code
     */
    private static final boolean DEBUGGING = false;

    private static final int FIRST_COPYRIGHT_YEAR = 2002;

    /**
     * undisplayed copyright notice
     *
     * @noinspection UnusedDeclaration
     */
    private static final String EMBEDDED_COPYRIGHT =
            "Copyright: (c) 2002-2013 Roedy Green, Canadian Mind Products, http://mindprod.com";

    /**
     * @noinspection UnusedDeclaration
     */
    private static final String RELEASE_DATE = "2013-02-19";

    /**
     * embedded version string.
     *
     * @noinspection UnusedDeclaration
     */
    private static final String VERSION_STRING = "1.8";

    /**
     * encoding for IBM437
     */
    public static final Charset IBM437Charset = Charset.forName( "IBM437" );

    /**
     * encoding for iso-8859-1
     */
    public static final Charset ISO88591Charset = Charset.forName( "ISO-8859-1" );

    /**
     * encoding for UTF-16
     */
    public static final Charset UTF16Charset = Charset.forName( "UTF-16" );

    /**
     * encoding for UTF-8
     */
    public static final Charset UTF8Charset = Charset.forName( "UTF-8" );

    /**
     * encoding for code page 1252
     */
    public static final Charset WINDOWS1252Charset = Charset.forName( "windows-1252" );

    // -------------------------- PUBLIC STATIC METHODS --------------------------


    /**
     * Create a temporary file, Slightly smarter version of File.createTempFile
     *
     * @param prefix beginning letters of filename
     * @param suffix ending letters of filename. e.g. ".temp". null means ".tmp",
     *               including any dot.
     * @param near   directory where to put file, or file to place this temp file near in the same directory. null means
     *               put the temp file in the default temp dir, e.g. C:\Users\Roedy\AppData\Local\Temp\.
     *
     * @return A temporary file. It will not automatically delete on program completion, however.
     * @throws java.io.IOException when cannot create file.
     */
    public static File createTempFile( final String prefix,
                                       final String suffix,
                                       final File near ) throws IOException
        {
        if ( near != null )
            {
            if ( near.isDirectory() )
                {
                return File.createTempFile( prefix, suffix, near );
                }
            else if ( near.isFile() )
                {
                String parent = near.getParent();
                if ( parent != null )
                    {
                    File dir = new File( parent );
                    if ( dir.isDirectory() )
                        {
                        return File.createTempFile( prefix, suffix, dir );
                        }
                    }
                }
            }
        // put in C:\Users\Roedy\AppData\Local\Temp\  default temp dir.
        return File.createTempFile( prefix, suffix );
        }

    /**
     * Get all text in a file.
     *
     * @param fromFile file where to get the text.
     *
     * @return all the text in the File.
     * @throws IOException when cannot access file.
     * @noinspection WeakerAccess
     */
    public static String readEntireFile( File fromFile ) throws IOException
        {
        // decode with default encoding and return entire file as one big string
        return new String( readEntireFileAsBytes( fromFile ) );
        }// end readEntireFile

    /**
     * Get all text in a file.
     *
     * @param fromFile Name of file where to get the text.
     *
     * @return all the text in the File.
     * @throws IOException when cannot access file.
     */
    public static String readEntireFile( String fromFile ) throws IOException
        {
        return readEntireFile( new File( fromFile ) );
        }// end readEntireFile

    /**
     * Get all text in a file.
     *
     * @param fromFile file where to get the text.
     * @param encoding name of the encoding to use to translate the bytes in the file into chars e.g. "windows-1252"
     *                 "UTF-8" "UTF-16"
     *
     * @return all the text in the File.
     * @throws IOException when cannot access file.
     * @noinspection WeakerAccess
     */
    public static String readEntireFile( File fromFile,
                                         Charset encoding ) throws IOException
        {
        // decode with encoding and return entire file as one big string
        return new String( readEntireFileAsBytes( fromFile ), encoding );
        }// end readEntireFile

    /**
     * read file of bytes without conversion
     *
     * @param fromFile file to read
     *
     * @return byte array representing whole file
     * @throws IOException when cannot access file.
     * @noinspection WeakerAccess
     */
    public static byte[] readEntireFileAsBytes( File fromFile ) throws IOException
        {
        int size = ( int ) fromFile.length();
        FileInputStream fis = new FileInputStream( fromFile );
        // R E A D
        byte[] rawContents = new byte[ size ];
        int bytesRead = fis.read( rawContents );
        if ( bytesRead != size )
            {
            throw new IOException( "error: problems reading file " + fromFile );
            }
        // C L O S E
        fis.close();
        return rawContents;
        }

    /**
     * Write all the text in a file
     *
     * @param toFile file where to write the text
     * @param text   Text to write
     *
     * @throws IOException when cannot access file.
     * @noinspection WeakerAccess
     */
    public static void writeEntireFile( File toFile,
                                        String text ) throws IOException
        {
        // default encoding
        FileWriter fw = new FileWriter( toFile );
        // W R I T E
        fw.write( text ); // one huge unbuffered write, default encoding
        // C L O S E
        fw.close();
        }// end writeEntireFile

    /**
     * Write all the text in a file
     *
     * @param toFile   file where to write the text, embedded line endings will be left as is.
     * @param text     Text to write
     * @param encoding name of the encoding to use to translate chars to bytes e.g. "windows-1252" "UTF-8" "UTF-16"
     *
     * @throws IOException when cannot access file.
     * @throws java.io.UnsupportedEncodingException
     *                     If the named encoding is not supported
     * @noinspection WeakerAccess
     */
    public static void writeEntireFile( File toFile,
                                        String text,
                                        Charset encoding ) throws IOException
        {
        // supplied encoding
        FileOutputStream fos = new FileOutputStream( toFile );
        OutputStreamWriter osw = new OutputStreamWriter( fos, encoding );
        // W R I T E
        osw.write( text );
        // C L O S E
        osw.close();
        }// end writeEntireFile

    /**
     * read file of bytes without conversion
     *
     * @param toFile   file to write to
     * @param contents bytes to write to file
     *
     * @throws IOException when cannot access file.
     * @noinspection WeakerAccess
     */
    public static void writeEntireFileAsBytes( File toFile, byte[] contents ) throws IOException
        {
        FileOutputStream fos = new FileOutputStream( toFile );
        fos.write( contents );
        // C L O S E
        fos.close();
        }

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * all methods are static, so hidden constructor
     */
    private HunkIO()
        {
        }

    // --------------------------- main() method ---------------------------

    /**
     * debugging harness for HunkIO.
     *
     * @param args not used.
     */
    public static void main( String[] args )
        {
        if ( DEBUGGING )
            {
            try
                {
                writeEntireFile( new File( "C:/temp/temp.txt" ),
                        "abcedefghijklmnopqrstuvwxyz",
                        UTF8Charset );
                String result = readEntireFile( new File( "C:/temp/temp.txt" ), UTF8Charset );
                System.out.println( result );
                }
            catch ( IOException e )
                {
                System.err.println( e.getMessage() );
                }
            }
        }
    }// end HunkIO
